﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

namespace DatabaseCore.Migrations
{
    public partial class AddLecturers : Migration
    {
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateTable(
                name: "LecturerAcademicDegrees",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerAcademicDegreeName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    Order = table.Column<int>(type: "int", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_LecturerAcademicDegrees", x => x.Id);
                });

            migrationBuilder.CreateTable(
                name: "LecturerAcademicRanks",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerAcademicRankName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    Order = table.Column<int>(type: "int", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_LecturerAcademicRanks", x => x.Id);
                });

            migrationBuilder.CreateTable(
                name: "LecturerPosts",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerPostName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    Hours = table.Column<int>(type: "int", nullable: false),
                    Order = table.Column<int>(type: "int", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_LecturerPosts", x => x.Id);
                });

            migrationBuilder.CreateTable(
                name: "Lecturers",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    UserId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerPostId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerPostRate = table.Column<decimal>(type: "decimal(18,2)", nullable: false),
                    LecturerAcademicRankId = table.Column<Guid>(type: "uniqueidentifier", nullable: true),
                    LecturerAcademicDegreeId = table.Column<Guid>(type: "uniqueidentifier", nullable: true),
                    LastName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    FirstName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    Patronymic = table.Column<string>(type: "nvarchar(450)", nullable: true),
                    DateBirth = table.Column<DateTime>(type: "datetime2", nullable: false),
                    Address = table.Column<string>(type: "nvarchar(max)", nullable: false),
                    Email = table.Column<string>(type: "nvarchar(max)", nullable: false),
                    MobileNumber = table.Column<string>(type: "nvarchar(max)", nullable: false),
                    HomeNumber = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    Description = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    Photo = table.Column<byte[]>(type: "varbinary(max)", nullable: true),
                    GroupElectricalSafety = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_Lecturers", x => x.Id);
                    table.ForeignKey(
                        name: "FK_Lecturers_LecturerAcademicDegrees_LecturerAcademicDegreeId",
                        column: x => x.LecturerAcademicDegreeId,
                        principalTable: "LecturerAcademicDegrees",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Restrict);
                    table.ForeignKey(
                        name: "FK_Lecturers_LecturerAcademicRanks_LecturerAcademicRankId",
                        column: x => x.LecturerAcademicRankId,
                        principalTable: "LecturerAcademicRanks",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Restrict);
                    table.ForeignKey(
                        name: "FK_Lecturers_LecturerPosts_LecturerPostId",
                        column: x => x.LecturerPostId,
                        principalTable: "LecturerPosts",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_Lecturers_Users_UserId",
                        column: x => x.UserId,
                        principalTable: "Users",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "LecturerEmployeePosts",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    LecturerId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    EmployeePostId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    Rate = table.Column<decimal>(type: "decimal(18,2)", nullable: false),
                    IsInternalCombination = table.Column<bool>(type: "bit", nullable: false),
                    IsExternalCombination = table.Column<bool>(type: "bit", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_LecturerEmployeePosts", x => x.Id);
                    table.ForeignKey(
                        name: "FK_LecturerEmployeePosts_EmployeePosts_EmployeePostId",
                        column: x => x.EmployeePostId,
                        principalTable: "EmployeePosts",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_LecturerEmployeePosts_Lecturers_LecturerId",
                        column: x => x.LecturerId,
                        principalTable: "Lecturers",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateIndex(
                name: "IX_Employees_UserId",
                table: "Employees",
                column: "UserId");

            migrationBuilder.CreateIndex(
                name: "IX_LecturerAcademicDegrees_LecturerAcademicDegreeName",
                table: "LecturerAcademicDegrees",
                column: "LecturerAcademicDegreeName",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_LecturerAcademicRanks_LecturerAcademicRankName",
                table: "LecturerAcademicRanks",
                column: "LecturerAcademicRankName",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_LecturerEmployeePosts_EmployeePostId",
                table: "LecturerEmployeePosts",
                column: "EmployeePostId");

            migrationBuilder.CreateIndex(
                name: "IX_LecturerEmployeePosts_LecturerId",
                table: "LecturerEmployeePosts",
                column: "LecturerId");

            migrationBuilder.CreateIndex(
                name: "IX_LecturerPosts_LecturerPostName",
                table: "LecturerPosts",
                column: "LecturerPostName",
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_Lecturers_FirstName_LastName_Patronymic",
                table: "Lecturers",
                columns: new[] { "FirstName", "LastName", "Patronymic" },
                unique: true,
                filter: "[Patronymic] IS NOT NULL");

            migrationBuilder.CreateIndex(
                name: "IX_Lecturers_LecturerAcademicDegreeId",
                table: "Lecturers",
                column: "LecturerAcademicDegreeId");

            migrationBuilder.CreateIndex(
                name: "IX_Lecturers_LecturerAcademicRankId",
                table: "Lecturers",
                column: "LecturerAcademicRankId");

            migrationBuilder.CreateIndex(
                name: "IX_Lecturers_LecturerPostId",
                table: "Lecturers",
                column: "LecturerPostId");

            migrationBuilder.CreateIndex(
                name: "IX_Lecturers_UserId",
                table: "Lecturers",
                column: "UserId");

            migrationBuilder.AddForeignKey(
                name: "FK_Employees_Users_UserId",
                table: "Employees",
                column: "UserId",
                principalTable: "Users",
                principalColumn: "Id",
                onDelete: ReferentialAction.Cascade);
        }

        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropForeignKey(
                name: "FK_Employees_Users_UserId",
                table: "Employees");

            migrationBuilder.DropTable(
                name: "LecturerEmployeePosts");

            migrationBuilder.DropTable(
                name: "Lecturers");

            migrationBuilder.DropTable(
                name: "LecturerAcademicDegrees");

            migrationBuilder.DropTable(
                name: "LecturerAcademicRanks");

            migrationBuilder.DropTable(
                name: "LecturerPosts");

            migrationBuilder.DropIndex(
                name: "IX_Employees_UserId",
                table: "Employees");
        }
    }
}
