﻿using System;
using Microsoft.EntityFrameworkCore.Migrations;

namespace DatabaseCore.Migrations
{
    public partial class AddAcademicPlans : Migration
    {
        protected override void Up(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.CreateTable(
                name: "AcademicPlans",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    EducationDirectionId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    YearEntrance = table.Column<DateTime>(type: "datetime2", nullable: false),
                    YearFinish = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_AcademicPlans", x => x.Id);
                    table.ForeignKey(
                        name: "FK_AcademicPlans_EducationDirections_EducationDirectionId",
                        column: x => x.EducationDirectionId,
                        principalTable: "EducationDirections",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "TimeNorms",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    DisciplineBlockId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    TimeNormName = table.Column<string>(type: "nvarchar(450)", nullable: false),
                    TimeNormShortName = table.Column<string>(type: "nvarchar(max)", nullable: false),
                    TimeNormOrder = table.Column<int>(type: "int", nullable: false),
                    TimeNormEducationDirectionQualification = table.Column<int>(type: "int", nullable: true),
                    KindOfLoadName = table.Column<string>(type: "nvarchar(max)", nullable: false),
                    KindOfLoadAttributeName = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    KindOfLoadBlueAsteriskName = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    KindOfLoadBlueAsteriskAttributeName = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    KindOfLoadBlueAsteriskPracticName = table.Column<string>(type: "nvarchar(max)", nullable: true),
                    UseInLearningProgress = table.Column<bool>(type: "bit", nullable: false),
                    UseInSite = table.Column<bool>(type: "bit", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_TimeNorms", x => x.Id);
                    table.ForeignKey(
                        name: "FK_TimeNorms_DisciplineBlocks_DisciplineBlockId",
                        column: x => x.DisciplineBlockId,
                        principalTable: "DisciplineBlocks",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "AcademicPlanRecords",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    AcademicPlanId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    DisciplineId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    InDepartment = table.Column<bool>(type: "bit", nullable: false),
                    Semester = table.Column<int>(type: "int", nullable: false),
                    Zet = table.Column<int>(type: "int", nullable: false),
                    AcademicPlanRecordParentId = table.Column<Guid>(type: "uniqueidentifier", nullable: true),
                    IsParent = table.Column<bool>(type: "bit", nullable: false),
                    IsFacultative = table.Column<bool>(type: "bit", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_AcademicPlanRecords", x => x.Id);
                    table.ForeignKey(
                        name: "FK_AcademicPlanRecords_AcademicPlans_AcademicPlanId",
                        column: x => x.AcademicPlanId,
                        principalTable: "AcademicPlans",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_AcademicPlanRecords_Disciplines_DisciplineId",
                        column: x => x.DisciplineId,
                        principalTable: "Disciplines",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                });

            migrationBuilder.CreateTable(
                name: "AcademicPlanRecordTimeNormHours",
                columns: table => new
                {
                    Id = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    AcademicPlanRecordId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    TimeNormId = table.Column<Guid>(type: "uniqueidentifier", nullable: false),
                    PlanHours = table.Column<decimal>(type: "decimal(18,2)", nullable: false),
                    DateCreate = table.Column<DateTime>(type: "datetime2", nullable: false),
                    DateDelete = table.Column<DateTime>(type: "datetime2", nullable: true),
                    IsDeleted = table.Column<bool>(type: "bit", nullable: false)
                },
                constraints: table =>
                {
                    table.PrimaryKey("PK_AcademicPlanRecordTimeNormHours", x => x.Id);
                    table.ForeignKey(
                        name: "FK_AcademicPlanRecordTimeNormHours_AcademicPlanRecords_AcademicPlanRecordId",
                        column: x => x.AcademicPlanRecordId,
                        principalTable: "AcademicPlanRecords",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.Cascade);
                    table.ForeignKey(
                        name: "FK_AcademicPlanRecordTimeNormHours_TimeNorms_TimeNormId",
                        column: x => x.TimeNormId,
                        principalTable: "TimeNorms",
                        principalColumn: "Id",
                        onDelete: ReferentialAction.NoAction);
                });

            migrationBuilder.CreateIndex(
                name: "IX_AcademicPlanRecords_AcademicPlanId_DisciplineId_Semester",
                table: "AcademicPlanRecords",
                columns: new[] { "AcademicPlanId", "DisciplineId", "Semester" },
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_AcademicPlanRecords_DisciplineId",
                table: "AcademicPlanRecords",
                column: "DisciplineId");

            migrationBuilder.CreateIndex(
                name: "IX_AcademicPlanRecordTimeNormHours_AcademicPlanRecordId_TimeNormId",
                table: "AcademicPlanRecordTimeNormHours",
                columns: new[] { "AcademicPlanRecordId", "TimeNormId" },
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_AcademicPlanRecordTimeNormHours_TimeNormId",
                table: "AcademicPlanRecordTimeNormHours",
                column: "TimeNormId");

            migrationBuilder.CreateIndex(
                name: "IX_AcademicPlans_EducationDirectionId_YearEntrance",
                table: "AcademicPlans",
                columns: new[] { "EducationDirectionId", "YearEntrance" },
                unique: true);

            migrationBuilder.CreateIndex(
                name: "IX_TimeNorms_DisciplineBlockId",
                table: "TimeNorms",
                column: "DisciplineBlockId");

            migrationBuilder.CreateIndex(
                name: "IX_TimeNorms_TimeNormName",
                table: "TimeNorms",
                column: "TimeNormName",
                unique: true);
        }

        protected override void Down(MigrationBuilder migrationBuilder)
        {
            migrationBuilder.DropTable(
                name: "AcademicPlanRecordTimeNormHours");

            migrationBuilder.DropTable(
                name: "AcademicPlanRecords");

            migrationBuilder.DropTable(
                name: "TimeNorms");

            migrationBuilder.DropTable(
                name: "AcademicPlans");
        }
    }
}
