﻿using DatabaseCore.Models.Security;
using ModuleTools.Attributes;
using ModuleTools.Extensions;
using ModuleTools.Interfaces;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Runtime.Serialization;

namespace DatabaseCore.Models.Department
{
	/// <summary>
	/// Класс, описывающий студента кафедры
	/// </summary>
	[DataContract]
	[EntityDescription("Student", "Студент кафедры")]
	[EntityDependency("User", "UserId", "К какому пользователю относится студент")]
	[EntityDependency("StudentGroup", "StudentGroupId", "К какой группе относится студент")]
	public class Student : BaseEntity, IEntitySecurityExtenstion<Student>
    {
        [DataMember]
        [Required]
        [MapConfiguration("UserId")]
        public Guid UserId { get; set; }

        [DataMember]
        [MapConfiguration("StudentGroupId")]
        public Guid? StudentGroupId { get; set; }

        [DataMember]
        [Required]
        [MapConfiguration("Iduniv")]
        public string Iduniv { get; set; }

        [DataMember]
        [Required]
        [MapConfiguration("NumberOfBook")]
        public string NumberOfBook { get; set; }

        [DataMember]
        [Required]
        [MapConfiguration("LastName")]
        public string LastName { get; set; }

        [DataMember]
        [Required]
        [MapConfiguration("FirstName")]
        public string FirstName { get; set; }

        [DataMember]
        [MapConfiguration("Patronymic")]
        public string Patronymic { get; set; }

        [DataMember]
        [MapConfiguration("Email")]
        public string Email { get; set; }

        [DataMember]
        [MapConfiguration("Description")]
        public string Description { get; set; }

        [Required(ErrorMessage = "required")]
        [MapConfiguration("StudentState")]
        public int StudentState { get; set; }

        [DataMember]
        [MapConfiguration("Photo")]
        public byte[] Photo { get; set; }

        [DataMember]
        [MapConfiguration("IsSteward")]
        public bool IsSteward { get; set; }

        //-------------------------------------------------------------------------

        public virtual User User { get; set; }

        public virtual StudentGroup StudentGroup { get; set; }

        //-------------------------------------------------------------------------

        [ForeignKey("StudentId")]
        public virtual List<OrderStudentRecord> OrderStudentRecords { get; set; }

        //-------------------------------------------------------------------------

        public Student SecurityCheck(Student entity, bool allowFullData)
        {
            if (!allowFullData)
            {
                entity.NumberOfBook = Guid.NewGuid().ToString();
                entity.Email = "скрыто";
                entity.Photo = null;
            }

            return entity;
        }

        public override string ToString() =>
            $"{LastName}{(FirstName.IsNotEmpty() ? $" {FirstName[0]}." : string.Empty)}{(Patronymic.IsNotEmpty() ? $"{Patronymic[0]}." : string.Empty)}";
    }
}