﻿using CoreModels.Enums.Department;
using CoreModels.ModelsDepartment;
using ModuleTools.Attributes;
using ModuleTools.Enums;
using ModuleTools.ViewModels;
using System;

namespace DepartmentBusinessLogic.ViewModels
{
    /// <summary>
    /// Список часов по нагрузке для записи учебного плана
    /// </summary>
    public class AcademicPlanRecordTimeNormHourListViewModel : ListViewModel<AcademicPlanRecordTimeNormHourViewModel> { }

	/// <summary>
	/// Элемент часов по нагрузке для записи учебного плана
	/// </summary>
	[ViewModelControlElementClass()]
	public class AcademicPlanRecordTimeNormHourViewModel : ElementViewModel, IAcademicPlanRecordTimeNormHourModel
	{
		[ViewModelControlElementProperty("Запись учебного плана", ControlType.ControlGuid, MustHaveValue = true, ReadOnly = false, ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlAcademicPlanRecordList, DepartmentWindowsDesktop")]
		public Guid AcademicPlanRecordId { get; set; }

		[ViewModelControlListProperty("Дисциплина")]
		[MapConfiguration("AcademicPlanRecord.Discipline.ToString", IsDifficle = true)]
		public string DisciplineName { get; set; }

		[MapConfiguration("AcademicPlanRecord.Semester", IsDifficle = true)]
		public Semester Semester { get; set; }

		[ViewModelControlListProperty("Семестр", ColumnWidth = 80)]
		public string SemesterTitle => Semester.ToString("G");

		[ViewModelControlElementProperty("Норма времени", ControlType.ControlGuid, MustHaveValue = true, ReadOnly = false, ControlTypeObject = "DepartmentWindowsDesktop.EntityControls.ControlTimeNormList, DepartmentWindowsDesktop")]
		public Guid TimeNormId { get; set; }

		[ViewModelControlListProperty("Норма времени")]
		[MapConfiguration("TimeNorm.ToString", IsDifficle = true)]
		public string TimeNormName { get; set; }

		[ViewModelControlListProperty("Часы", ColumnWidth = 80)]
		[ViewModelControlElementProperty("Часы", ControlType.ControlDecimal, MustHaveValue = true)]
		public decimal PlanHours { get; set; }

		public override string ToString() => $"{DisciplineName}({Semester}) - {TimeNormName}";
	}
}